namespace Commonmark {

    /**
     * Abstract visitor that visits all children by default.
     *
     * Can be used to only process certain nodes. If you override a method and want
     * visiting to descend into children, call #visit_children.
     */
    public abstract class AbstractVisitor : GLib.Object, Visitor {

        public virtual void visit(Node node) {
            if (node is BlockQuote              ||
                node is BulletList              ||
                node is Code                    ||
                node is CustomBlock             ||
                node is CustomNode              ||
                node is Document                ||
                node is Emphasis                ||
                node is FencedCodeBlock         ||
                node is HardLineBreak           ||
                node is Heading                 ||
                node is HtmlBlock               ||
                node is HtmlInline              ||
                node is Image                   ||
                node is IndentedCodeBlock       ||
                node is Link                    ||
                node is LinkReferenceDefinition ||
                node is ListItem                ||
                node is OrderedList             ||
                node is Paragraph               ||
                node is SoftLineBreak           ||
                node is StrongEmphasis          ||
                node is Text                    ||
                node is ThematicBreak) {
                visit_children(node);
            }
        }

        /**
         * Visit the child nodes.
         *
         * @param parent the parent node whose children should be visited
         */
        protected virtual void visit_children(Node parent) {
            Node node = parent.first_child;
            while (node != null) {
                /* A subclass of this visitor might modify the node, resulting
                   in next returning a different node or no node after visiting
                   it. So get the next node before visiting. */
                Node next = node.next;
                node.accept(this);
                node = next;
            }
        }
    }
}
