namespace Commonmark {

    /**
     * Attempt to parse an autolink (URL or email in pointy brackets).
     */
    public class AutolinkInlineParser : GLib.Object, InlineContentParser {

        private static GLib.Regex URI;

        private static GLib.Regex EMAIL;

        public ParsedInline try_parse(InlineParserState inline_parser_state) {
            var scanner = inline_parser_state.scanner();
            scanner.next();
            var text_start = scanner.position();
            if (scanner.find('>') > 0) {
                var text_source = scanner.get_source(text_start,
                                                     scanner.position());
                var content = text_source.get_content();
                scanner.next();

                string destination = null;
                if (URI.match(content)) {
                    destination = content;
                } else if (EMAIL.match(content)) {
                    destination = "mailto:" + content;
                }

                if (destination != null) {
                    var link = new Link.full(destination, null);
                    var text = new Text.from_literal(content);
                    text.source_spans = text_source.get_source_spans();
                    link.append_child(text);
                    return ParsedInline.from(link, scanner.position());
                }
            }
            return ParsedInline.none();
        }

        static construct {
            try {
                var r = "^[a-zA-Z][a-zA-Z0-9.+-]{1,31}:[^<>]*$";
                URI = new GLib.Regex(r);
                EMAIL = new GLib.Regex("^([a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+" +
                                       "@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}" +
                                       "[a-zA-Z0-9])?(?:\\.[a-zA-Z0-9](?:" +
                                       "[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*)$");
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
        }
    }
}
