namespace Commonmark {

    /**
     * Parse a backslash-escaped special character, adding either the escaped
     * character, a hard line break (if the backslash is followed by a newline),
     * or a literal backslash to the block's children.
     */
    public class BackslashInlineParser : GLib.Object, InlineContentParser {

        private static GLib.Regex ESCAPABLE;

        public ParsedInline try_parse(InlineParserState inline_parser_state) {
            var scanner = inline_parser_state.scanner();
            /* Backslash */
            scanner.next();

            unichar next = scanner.peek();
            if (next == '\n') {
                scanner.next();
                return ParsedInline.from(new HardLineBreak(),
                                         scanner.position());
            } else if (ESCAPABLE.match("%c".printf((int)next))) {
                scanner.next();
                var c = "%c".printf((int)next);
                return ParsedInline.from(new Text.from_literal(c),
                                         scanner.position());
            } else {
                return ParsedInline.from(new Text.from_literal("\\"),
                                         scanner.position());
            }
        }

        static construct {
            try {
                ESCAPABLE = new GLib.Regex("^" + Escaping.ESCAPABLE);
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
        }
    }
}
