namespace Commonmark {

    /**
     * Attempt to parse backticks, returning either a backtick code span or a
     * literal sequence of backticks.
     */
    public class BackticksInlineParser : GLib.Object, InlineContentParser {

        public ParsedInline try_parse(InlineParserState inline_parser_state) {
            var scanner = inline_parser_state.scanner();
            var start = scanner.position();
            int opening_ticks = scanner.match_multiple('`');
            var after_opening = scanner.position();

            while (scanner.find('`') > 0) {
                var before_closing = scanner.position();
                int count = scanner.match_multiple('`');
                if (count == opening_ticks) {
                    var node = new Code();

                    var content =
                        scanner.get_source(after_opening, before_closing)
                        .get_content();
                    content = content.replace("\n", " ");

                    /* spec: If the resulting string both begins and ends with a
                       space character, but does not consist entirely of space
                       characters, a single space character is removed from the
                       front and back.*/
                    if (content.length >= 3 &&
                        content.get(0) == ' ' &&
                        content.get(content.length - 1) == ' ' &&
                        Parsing.has_non_space(content)) {
                        content = content.slice(1, content.length - 1);
                    }

                    node.literal = content;
                    return ParsedInline.from(node, scanner.position());
                }
            }

            /* If we got here, we didn't find a matching closing backtick
             * sequence. */
            var source = scanner.get_source(start, after_opening);
            var text = new Text.from_literal(source.get_content());
            return ParsedInline.from(text, after_opening);
        }
    }
}
