namespace Commonmark {

    /**
     * Parser for a specific block node.
     *
     * Implementations should subclass AbstractBlockParser instead of
     * implementing this directly.
     */
    public interface BlockParser : GLib.Object {

        /**
         * Return true if the block that is parsed is a container (contains
         * other blocks), or false if it's a leaf.
         */
        public abstract bool is_container();

        /**
         * Return true if the block can have lazy continuation lines.
         *
         * Lazy continuation lines are lines that were rejected by this
         * try_continue(ParserState) but didn't match any other block parsers
         * either.
         *
         * If true is returned here, those lines will get added via
         * add_line(SourceLine). For false, the block is closed instead.
         */
        public abstract bool can_have_lazy_continuation_lines();

        public abstract bool can_contain(Block child_block);

        public abstract Block get_block();

        public abstract BlockContinue try_continue(ParserState parser_state);

        public abstract void add_line(SourceLine line);

        /**
         * Add a source span of the currently parsed block. The default
         * implementation in AbstractBlockParser adds it to the block. Unless
         * you have some complicated parsing where you need to check source
         * positions, you don't need to override this.
         */
        public abstract void add_source_span(SourceSpan source_span);

        public abstract void close_block();

        public abstract void parse_inlines(InlineParser inline_parser);
    }
}
