namespace Commonmark {

    public class BlockQuoteParser : AbstractBlockParser {

        private BlockQuote block;

        construct {
            block = new BlockQuote();
        }

        public override bool is_container() {
            return true;
        }

        public override bool can_contain(Block block) {
            return true;
        }

        public override Block get_block() {
            return block;
        }

        public override BlockContinue
            try_continue(ParserState state) {
            int next_non_space = state.get_next_non_space_index();
            if (is_marker(state, next_non_space)) {
                int new_column = state.get_column() + state.get_indent() + 1;
                /* optional following space or tab */
                if (Parsing.is_space_or_tab(state.get_line().content,
                                            next_non_space + 1)) {
                    new_column++;
                }
                return BlockContinue.at_column(new_column);
            } else {
                return BlockContinue.none();
            }
        }

        private static bool
            is_marker(ParserState state, int index) {
            var line = state.get_line().content;
            return state.get_indent() < Parsing.CODE_BLOCK_INDENT &&
                index < line.length && line.get(index) == '>';
        }

        public class Factory : AbstractBlockParserFactory {

            public override BlockStart
                try_start(ParserState state,
                          MatchedBlockParser matched_block_parser) {
                int next_non_space = state.get_next_non_space_index();
                if (is_marker(state, next_non_space)) {
                    int new_column = state.get_column() +
                        state.get_indent() + 1;
                    /* optional following space or tab */
                    if (Parsing.is_space_or_tab(state.get_line().content,
                                                next_non_space + 1)) {
                        new_column++;
                    }
                    return BlockStart.from(new BlockQuoteParser())
                        .at_column(new_column);
                } else {
                    return BlockStart.none();
                }
            }
        }
    }
}
