namespace Commonmark {

    /**
     * Opening bracket for links (''['') or images (''!['').
     */
    public class Bracket : GLib.Object {

        public Text node;

        /**
         * The position of the marker for the bracket (''['' or ''!['')
         */
        public Position marker_position;

        /**
         * The position of the content (after the opening bracket)
         */
        public Position content_position;

        /**
         * Whether this is an image or link.
         */
        public bool _image;

        /**
         * Previous bracket.
         */
        public Bracket previous;

        /**
         * Previous delimiter (emphasis, etc) before this bracket.
         */
        public Delimiter previous_delimiter;

        /**
         * Whether this bracket is allowed to form a link/image (also known as
         * "active").
         */
        public bool allowed;

        /**
         * Whether there is an unescaped bracket (opening or closing) anywhere
         * after this opening bracket.
         */
        public bool bracket_after;

        public static Bracket link(Text node,
                                   Position marker_position,
                                   Position content_position,
                                   Bracket? previous,
                                   Delimiter? previous_delimiter) {
            return new Bracket(node, marker_position, content_position,
                               previous, previous_delimiter, false);
        }

        public static Bracket image(Text node,
                                    Position marker_position,
                                    Position content_position,
                                    Bracket? previous,
                                    Delimiter? previous_delimiter) {
            return new Bracket(node, marker_position, content_position,
                               previous, previous_delimiter, true);
        }

        private Bracket(Text node,
                        Position marker_position,
                        Position content_position,
                        Bracket? previous,
                        Delimiter? previous_delimiter,
                        bool image) {
            this.node = node;
            this.marker_position = marker_position;
            this.content_position = content_position;
            this._image = image;
            this.previous = previous;
            this.previous_delimiter = previous_delimiter;
            allowed = true;
        }
    }
}
