namespace Commonmark {

    /**
     * Allows http, https and mailto protocols for url.
     *
     * Also allows protocol relative urls, and relative urls.
     *
     * Implementation based on
     * [[https://github.com/OWASP/java-html-sanitizer/blob/main/src/main/java/org/owasp/html/FilterUrlByProtocolAttributePolicy.java]]
     */
    public class DefaultUrlSanitizer : GLib.Object, UrlSanitizer {

        private Gee.Set<string> protocols;

        public DefaultUrlSanitizer() {
            protocols = new Gee.HashSet<string>();
            protocols.add("http");
            protocols.add("https");
            protocols.add("mailto");
        }

        public DefaultUrlSanitizer
            .from_collection(Gee.Collection<string> protocols) {
            this.protocols = new Gee.HashSet<string>();
            this.protocols.add_all(protocols);
        }

        public string sanitize_link_url(string _url) {
            var url = strip_html_spaces(_url);
            for (int i = 0, n = url.length; i < n; ++i) {
                switch (url.get(i)) {
                case '/':
                case '#':
                case '?':  /* No protocol. */
                    break;
                case ':':
                    var protocol = url.slice(0, i).down();
                    if (!protocols.contains(protocol)) {
                        return "";
                    }
                    break;
                }
            }
            return url;
        }


        public string sanitize_image_url(string url) {
            return sanitize_link_url(url);
        }

        private string strip_html_spaces(string s) {
            int i = 0, n = s.length;
            for (; n > i; --n) {
                if (!is_html_space(s.get(n - 1))) {
                    break;
                }
            }
            for (; i < n; ++i) {
                if (!is_html_space(s.get(i))) {
                    break;
                }
            }
            if (i == 0 && n == s.length) {
                return s;
            }
            return s.substring(i, n);
        }

        private bool is_html_space(int ch) {
            switch (ch) {
            case ' ':
            case '\t':
            case '\n':
            case '\x0c':
            case '\r':
                return true;
            default:
                return false;

            }
        }
    }
}
