namespace Commonmark {

    /**
     * Custom delimiter processor for additional delimiters besides ''_'' and
     * ''*''.
     *
     * Note that implementations of this need to be thread-safe, the same
     * instance may be used by multiple parsers.
     */
    public interface DelimiterProcessor : GLib.Object {

        /**
         * @return the character that marks the beginning of a delimited node, must
         * not clash with any built-in special characters
         */
        public abstract unichar get_opening_character();

        /**
         * @return the character that marks the the ending of a delimited node,
         * must not clash with any built-in special characters. Note that for a
         * symmetric delimiter such as "*", this is the same as the opening.
         */
        public abstract unichar get_closing_character();

        /**
         * Minimum number of delimiter characters that are needed to activate
         * this. Must be at least 1.
         */
        public abstract int get_min_length();

        /**
         * Process the delimiter runs.
         *
         * The processor can examine the runs and the nodes and decide if it
         * wants to process or not. If not, it should not change any nodes and
         * return 0. If yes, it should do the processing (wrapping nodes, etc)
         * and then return how many delimiters were used.
         *
         * Note that removal (unlinking) of the used delimiter {@link Node.Text}
         * nodes is done by the caller.
         *
         * @param opening_run the opening delimiter run
         * @param closing_run the closing delimiter run
         * @return how many delimiters were used; must not be greater than
         * length of either opener or closer
         */
        public abstract int process(DelimiterRun opening_run,
                                    DelimiterRun closing_run);
    }
}
