namespace Commonmark {

    /**
     * A delimiter run is one or more of the same delimiter character,
     * e.g. ***.
     */
    public interface DelimiterRun {

        /**
         * @return whether this can open a delimiter
         */
        public abstract bool can_open();

        /**
         * @return whether this can close a delimiter
         */
        public abstract bool can_close();

        /**
         * @return the number of characters in this delimiter run (that are left
         * for processing)
         */
        public abstract int length();

        /**
         * @return the number of characters originally in this delimiter run; at
         * the start of processing, this is the same as #length()
         */
        public abstract int original_length();

        /**
         * @return the innermost opening delimiter, e.g. for *** this is the
         * last *
         */
        public abstract Text get_opener();

        /**
         * @return the innermost closing delimiter, e.g. for *** this is the
         * first *
         */
        public abstract Text get_closer();

        /**
         * Get the opening delimiter nodes for the specified length of
         * delimiters. Length must be between 1 and #length().
         *
         * For example, for a delimiter run ***, calling this with 1 would
         * return the last *.  Calling it with 2 would return the second last *
         * and the last *.
         */
        public abstract Gee.Iterable<Text> get_openers(int length);

        /**
         * Get the closing delimiter nodes for the specified length of
         * delimiters. Length must be between 1 and #length().
         *
         * For example, for a delimiter run ***, calling this with 1 would
         * return the first *.  Calling it with 2 would return the first * and
         * the second *.
         */
        public abstract Gee.Iterable<Text> get_closers(int length);
    }
}
