namespace Commonmark {

    /**
     * Delimiter (emphasis, strong emphasis or custom emphasis).
     */
    public class Delimiter : GLib.Object, DelimiterRun {

        public Gee.List<Text> characters;
        public unichar delimiter_char;
        private int _original_length;

        /* Can open emphasis, see spec. */
        private bool _can_open;

        /* Can close emphasis, see spec. */
        private bool _can_close;

        public Delimiter previous;
        public Delimiter next;

        public Delimiter(Gee.List<Text> characters,
                         unichar delimiter_char,
                         bool can_open, bool can_close,
                         Delimiter? previous) {
            this.characters = characters;
            this.delimiter_char = delimiter_char;
            this._can_open = can_open;
            this._can_close = can_close;
            this.previous = previous;
            this._original_length = characters.size;
        }

        /**
         * @return whether this can open a delimiter
         */
        public bool can_open() {
            return _can_open;
        }

        /**
         * @return whether this can close a delimiter
         */
        public bool can_close() {
            return _can_close;
        }

        /**
         * @return the number of characters originally in this delimiter run; at
         * the start of processing, this is the same as #length()
         */
        public int original_length() {
            return _original_length;
        }

        public int length() {
            return characters.size;
        }

        public Text get_opener() {
            return characters.get(characters.size - 1);
        }

        public Text get_closer() {
            return characters.get(0);
        }

        public Gee.Iterable<Text> get_openers(int length) {
            if (!(length >= 1 && length <= this.length()))
                GLib.error("length must be between 1 and %d, was %d",
                           this.length(), length);

            return characters.slice(characters.size - length, characters.size);
        }

        public Gee.Iterable<Text> get_closers(int length) {
            if (!(length >= 1 && length <= this.length()))
                GLib.error("length must be between 1 and %d, was %d",
                           this.length(), length);

            return characters.slice(0, length);
        }
    }
}
