namespace Commonmark {

    public abstract class EmphasisDelimiterProcessor :
    GLib.Object, DelimiterProcessor {

        private unichar delimiter_char;

        protected EmphasisDelimiterProcessor(unichar delimiter_char) {
            this.delimiter_char = delimiter_char;
        }

        public unichar get_opening_character() {
            return delimiter_char;
        }

        public unichar get_closing_character() {
            return delimiter_char;
        }

        public int get_min_length() {
            return 1;
        }

        public int process(DelimiterRun opening_run, DelimiterRun closing_run) {
            /* "multiple of 3" rule for internal delimiter runs */
            if ((opening_run.can_close() || closing_run.can_open()) &&
                closing_run.original_length() % 3 != 0 &&
                (opening_run.original_length() +
                 closing_run.original_length()) % 3 == 0) {
                return 0;
            }

            int used_delimiters;
            Node emphasis;

            /* calculate actual number of delimiters used from this closer */
            if (opening_run.length() >= 2 && closing_run.length() >= 2) {
                used_delimiters = 2;
                var sb = new StringBuilder();
                sb.append_unichar(delimiter_char);
                sb.append_unichar(delimiter_char);
                emphasis = new StrongEmphasis.from_string(sb.str);
            } else {
                used_delimiters = 1;
                var sb = new StringBuilder();
                sb.append_unichar(delimiter_char);
                emphasis = new Emphasis.from_string(sb.str);
            }

            var source_spans = SourceSpans.empty();
            source_spans.add_all_from(opening_run.get_openers(used_delimiters));

            var opener = opening_run.get_opener();
            foreach (var node in Nodes.between(opener,
                                               closing_run.get_closer())) {
                emphasis.append_child(node);
                source_spans.add_all(node.source_spans);
            }

            source_spans.add_all_from(closing_run.get_closers(used_delimiters));

            emphasis.source_spans = source_spans.get_source_spans();
            opener.insert_after(emphasis);

            return used_delimiters;
        }
    }
}
