namespace Commonmark {

    /**
     * Attempts to parse a HTML entity or numeric character reference.
     */
    public class EntityInlineParser : GLib.Object, InlineContentParser {

        private static AsciiMatcher HEX;
        private static AsciiMatcher DEC;
        private static AsciiMatcher ENTITY_START;
        private static AsciiMatcher ENTITY_CONTINUE;

        public ParsedInline try_parse(InlineParserState inline_parser_state) {
            var scanner = inline_parser_state.scanner();
            var start = scanner.position();
            /* Skip `&` */
            scanner.next();

            unichar c = scanner.peek();
            if (c == '#') {
                /* Numeric */
                scanner.next();
                if (scanner.next_char('x') || scanner.next_char('X')) {
                    int digits = scanner.match(HEX);
                    if (1 <= digits && digits <= 6 && scanner.next_char(';')) {
                        return entity(scanner, start);
                    }
                } else {
                    int digits = scanner.match(DEC);
                    if (1 <= digits && digits <= 7 && scanner.next_char(';')) {
                        return entity(scanner, start);
                    }
                }
            } else if (ENTITY_START.matches(c)) {
                scanner.match(ENTITY_CONTINUE);
                if (scanner.next_char(';')) {
                    return entity(scanner, start);
                }
            }

            return ParsedInline.none();
        }

        private ParsedInline entity(Scanner scanner, Position start) {
            var text = scanner.get_source(start, scanner.position())
                .get_content();
            var h5 = Html5Entities.entity_to_string(text);
            var t = new Text.from_literal(h5);
            return ParsedInline.from(t, scanner.position());
        }

        static construct {
            HEX = AsciiMatcher.builder().range('0', '9').range('A', 'F')
                .range('a', 'f').build();
            DEC = AsciiMatcher.builder().range('0', '9').build();
            ENTITY_START = AsciiMatcher.builder().range('A', 'Z')
                .range('a', 'z').build();
            ENTITY_CONTINUE = ENTITY_START.new_builder()
                .range('0', '9').build();
        }
    }
}
