namespace Commonmark {

    public class Escaping : GLib.Object {

        public const string ESCAPABLE =
            "[!\"#$%&\'()*+,./:;<=>?@\\[\\\\\\]^_`{|}~-]";

        private static GLib.Regex WHITESPACE;

        public static string escape_html(string input) {
            return (input.replace("&", "&amp;")
                    .replace("<", "&lt;")
                    .replace(">", "&gt;)")
                    .replace("\"", "&quot;)"));
        }

        /**
         * Replace entities and backslash escapes with literal characters.
         */
        public static string unescape_string(string s) {
            if (s.has_prefix("\\"))
                return s.slice(1, s.length);
            if (s.has_prefix("&")) {
                int i = s.index_of(";");
                if (i++ == -1)
                    return s;
                var r = s.slice(0, i);
                var t =  Html5Entities.entity_to_string(r);
                return s.replace(r, t);
            }
            return s;
        }

        public static string percent_encode_url(string s) {
            try {
                var uri = GLib.Uri.parse(s, GLib.UriFlags.SCHEME_NORMALIZE);
                return uri.to_string();
            } catch (GLib.Error error) {
                return s;
            }
        }

        public static string normalize_label_content(string input) {
            try {
                return WHITESPACE.replace(input, input.length, 0, " ");
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
        }

        static construct {
            try {
                WHITESPACE = new GLib.Regex("[ \t\r\n]+");
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
        }
    }
}
