namespace Commonmark {

    public interface HtmlNodeRendererContext : GLib.Object {

        /**
         * @param url to be encoded
         * @return an encoded URL (depending on the configuration)
         */
        public abstract string encode_url(string url);

        /**
         * Let extensions modify the HTML tag attributes.
         *
         * @param node the node for which the attributes are applied
         * @param tagName the HTML tag name that these attributes are for
         * (e.g. h1, pre, code).
         * @param attributes the attributes that were calculated by the renderer
         * @return the extended attributes with added/updated/removed entries
         */
        public abstract Gee.Map<string, string>
        extend_attributes(Node node, string tagName,
                          Gee.Map<string, string> attributes);

        /**
         * @return the HTML writer to use
         */
        public abstract HtmlWriter get_writer();

        /**
         * @return HTML that should be rendered for a soft line break
         */
        public abstract string get_softbreak();

        /**
         * Render the specified node and its children using the configured
         * renderers. This should be used to render child nodes; be careful not
         * to pass the node that is being rendered, that would result in an
         * endless loop.
         *
         * @param node the node to render
         */
        public abstract void render(Node node);

        /**
         * @return whether HTML blocks and tags should be escaped or not
         */
        public abstract bool should_escape_html();

        /**
         * @return true if the UrlSanitizer should be used.
         */
        public abstract bool should_sanitize_urls();

        /**
         * @return Sanitizer to use for securing Link href and Image src if
         * #shouldSanitizeUrls() is true.
         */
        public abstract UrlSanitizer url_sanitizer();
    }
}
