namespace Commonmark {

    public class HtmlWriter : GLib.Object {

        private static Gee.Map<string, string>
        NO_ATTRIBUTES = new Gee.HashMap<string, string>();

        private GLib.OutputStream buffer;
        private unichar last_char = (unichar)0;

        public HtmlWriter(GLib.OutputStream _out) {
            if (_out == null)
                GLib.error("out must not be null");
            this.buffer = _out;
        }

        public void raw(string s) {
            append(s);
        }

        public void text(string text) {
            append(Escaping.escape_html(text));
        }

        public void tag(string name) {
            tag_attrs(name, NO_ATTRIBUTES);
        }

        public void tag_attrs(string name, Gee.Map<string, string> attrs) {
            tag_full(name, attrs, false);
        }

        public void tag_full(string name,
                             Gee.Map<string, string> attrs,
                             bool void_element) {
            append("<");
            append(name);
            if (attrs != null && !attrs.is_empty) {
                foreach (var attrib in attrs.entries) {
                    append(" ");
                    append(Escaping.escape_html(attrib.key));
                    append("=\"");
                    append(Escaping.escape_html(attrib.value));
                    append("\"");
                }
            }
            if (void_element) {
                append(" /");
            }

            append(">");
        }

        public void line() {
            if (last_char != 0 && last_char != '\n') {
                append("\n");
            }
        }

        protected void append(string s) {
            try {
                buffer.write(s.data);
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
            if (s.length != 0)
                last_char = s.get(s.length - 1);
        }
    }
}
