namespace Commonmark {

    public class Html5Entities : GLib.Object {

        private const string ENTITIES_RESOURCE =
            "resource:///org/commonmark/entities.properties";
        private static Gee.Map<string, string> NAMED_CHARACTER_REFERENCES;

        public static string entity_to_string(string input) {
            if (!input.has_prefix("&") || !input.has_suffix(";"))
                return input;

            string value = input.slice(1, input.length - 1);
            if (value.has_prefix("#")) {
                value = value.slice(1, value.length);
                int _base = 10;
                if (value.has_prefix("x") || value.has_prefix("X")) {
                    value = value.slice(1, value.length);
                    _base = 16;
                }

                int code_point;
                if (!int.try_parse (value, out code_point, null, _base))
                    return "\xff\xfd";
                if (code_point == 0)
                    return "\xff\xfd";
                return ((unichar)code_point).to_string();
            } else {
                string s = NAMED_CHARACTER_REFERENCES[value];
                if (s != null)
                    return s;
                else
                    return input;
            }
        }

        static construct {
            NAMED_CHARACTER_REFERENCES = new Gee.HashMap<string, string>();
            try {
                var file = GLib.File.new_for_uri(ENTITIES_RESOURCE);
                var dis = new GLib.DataInputStream(file.read());
                string line;
                while ((line = dis.read_line(null)) != null) {
                    if (line.length == 0)
                        continue;
                    var t = line.split("=");
                    NAMED_CHARACTER_REFERENCES[t[0]] = t[1];
                }
            } catch (GLib.Error error) {
                GLib.error(error.message);
            }
        }
    }
}
