namespace Commonmark {

    public class IndentedCodeBlockParser : AbstractBlockParser {

        private IndentedCodeBlock block;
        private Gee.List<string> lines;

        construct {
            block = new IndentedCodeBlock();
            lines = new Gee.ArrayList<string>();
        }

        public override Block get_block() {
            return block;
        }

        public override BlockContinue
            try_continue(ParserState state) {
            if (state.get_indent() >= Parsing.CODE_BLOCK_INDENT) {
                return BlockContinue.at_column(state.get_column() +
                                               Parsing.CODE_BLOCK_INDENT);
            } else if (state.is_blank()) {
                return BlockContinue.at_index(state.get_next_non_space_index());
            } else {
                return BlockContinue.none();
            }
        }

        public override void add_line(SourceLine line) {
            lines.add(line.content);
        }

        public override void close_block() {
            int last_non_blank = lines.size - 1;
            while (last_non_blank >= 0) {
                if (!Parsing.is_blank(lines.get(last_non_blank)))
                    break;
                last_non_blank--;
            }

            var sb = new StringBuilder();
            for (int i = 0; i < last_non_blank + 1; i++) {
                sb.append(lines.get(i));
                sb.append("\n");
            }

            block.literal = sb.str;
        }

        public class Factory : AbstractBlockParserFactory {

            public override BlockStart
                try_start(ParserState state,
                          MatchedBlockParser matched_block_parser) {
                /* An indented code block cannot interrupt a paragraph. */
                if (state.get_indent() >= Parsing.CODE_BLOCK_INDENT &&
                    !state.is_blank() &&
                    !(state.get_active_block_parser().get_block()
                      is Paragraph)) {
                    return BlockStart.from(
                        new IndentedCodeBlockParser()).at_column(
                            state.get_column() + Parsing.CODE_BLOCK_INDENT);
                } else {
                    return BlockStart.none();
                }
            }
        }
    }
}
