namespace Commonmark {

    public class LinkedMap<K, V> : Gee.AbstractMap<K, V> {

        private class Entry<K, V> : Gee.Map.Entry<K, V> {

            private K _key;
            public override K key { get { return _key; } }
            public override V value { get; set; }
            public override bool read_only { get { return false; } }

            public Entry(K key, V value) {
                _key = key;
                this.value = value;
            }
        }

        private class LinkedMapIterator<K, V> : GLib.Object, Gee.MapIterator<K, V> {

            private LinkedSet<Entry<K, V>> list;
            private Gee.Iterator<Entry<K, V>> iterator;

            public LinkedMapIterator(LinkedSet<Entry<K, V>> list) {
                this.list = list;
                iterator = list.iterator();
            }

            public override bool mutable { get { return false; } }
            public override bool read_only { get { return true; } }
            public override bool valid { get { return iterator.valid; } }

            public K get_key() {
                return iterator.get().key;
            }

            public V get_value() {
                return iterator.get().value;
            }

            public bool has_next() {
                return iterator.has_next();
            }

            public bool next() {
                return iterator.next();
            }

            public void set_value(V value) {
                GLib.error("Invalid operation: set_value");
            }

            public void unset() {
                GLib.error("Invalid operation: unset");
            }
        }


        public override Gee.Set<Gee.Map.Entry<K,V>> entries {
            owned get { return _entries; }
        }

        public override Gee.Set<K> keys { owned get { return _keys; } }

        public override bool read_only { get { return false; } }

        public override int size { get { return _entries.size; } }

        public override Gee.Collection<V> values {
            owned get { return values; }
        }

        private LinkedSet<K> _keys;
        private Gee.LinkedList<K> _values;
        private LinkedSet<Entry<K, V>> _entries;

        private Gee.EqualDataFunc<K> key_equals;
        private Gee.EqualDataFunc<V> value_equals;

        construct {
            _keys = new LinkedSet<K>();
            _values = new Gee.LinkedList<V>();
            _entries = new LinkedSet<Entry<K, V>>();
            key_equals = Gee.Functions.get_equal_func_for(typeof(K));
            value_equals = Gee.Functions.get_equal_func_for(typeof(V));
        }

        public override void clear() {
            _keys.clear();
            _values.clear();
            _entries.clear();
        }

        public override V @get(K key) {
            foreach (var entry in _entries) {
                if (key_equals(key, entry.key))
                    return entry.value;
            }
            return null;
        }

        public override bool has(K key, V value) {
            foreach (var entry in _entries)
                if (key_equals(key, entry.key) &&
                    value_equals(value, entry.value))
                    return true;
            return false;
        }

        public override bool has_key(K key) {
            foreach (var entry in _entries)
                if (key_equals(key, entry.key))
                    return true;
            return false;
        }

        public override Gee.MapIterator<K,V> map_iterator() {
            return new LinkedMapIterator<K, V>(_entries);
        }

        public override void @set(K key, V value) {
            bool found = false;
            foreach (var entry in _entries) {
                if (key_equals(key, entry.key)) {
                    entry.value = value;
                    found = true;
                }
            }
            if (found)
                return;
            var entry = new Entry<K, V>(key, value);
            _keys.add(key);
            _values.add(value);
            _entries.add(entry);
        }

        public override bool unset(K key, out V value = null) {
            value = null;
            Entry<K, V> to_remove = null;
            foreach (var entry in _entries) {
                if (key_equals(key, entry.key)) {
                    value = entry.value;
                    to_remove = entry;
                    break;
                }
            }
            if (to_remove == null)
                return false;
            _keys.remove(to_remove.key);
            _values.remove(to_remove.value);
            _entries.remove(to_remove);
            return true;
        }
    }
}
