namespace Commonmark {

    public class ListItemParser : AbstractBlockParser {

        private ListItem block;
        /**
         * Minimum number of columns that the content has to be indented (relative
         * to the containing block) to be part of this list item.
         */
        private int content_indent;
        private bool had_blank_line;

        public ListItemParser(int content_indent) {
            block = new ListItem();
            this.content_indent = content_indent;
        }

        public override bool is_container() {
            return true;
        }

        public override bool can_contain(Block child_block) {
            if (had_blank_line) {
                /* We saw a blank line in this list item, that means the list block
                   is loose.

                   spec: if any of its constituent list items directly contain two
                   block-level elements with a blank line between them */
                if (block.parent is ListBlock) {
                    ((ListBlock)block.parent).tight = false;
                }
            }
            return true;
        }

        public override Block get_block() {
            return block;
        }

        public override BlockContinue try_continue(ParserState state) {
            if (state.is_blank()) {
                if (block.first_child == null) {
                    /* Blank line after empty list item */
                    return BlockContinue.none();
                } else {
                    var active_block =
                        state.get_active_block_parser().get_block();
                    /* If the active block is a code block, blank lines in it
                       should not affect if the list is tight. */
                    had_blank_line =
                        (active_block is Paragraph ||
                         active_block is ListItem);
                    return BlockContinue.at_index(
                        state.get_next_non_space_index());
                }
            }

            if (state.get_indent() >= content_indent) {
                return BlockContinue.at_column(state.get_column() +
                                               content_indent);
            } else {
                /* Note: We'll hit this case for lazy continuation lines, they
                 * will get added later. */
                return BlockContinue.none();
            }
        }
    }
}
