namespace Commonmark {

    /**
     * Utility class for working with multiple Nodes.
     */
    public class Nodes {

        private Nodes() {}

        /**
         * The nodes between (not including) start and end.
         */
        public static Gee.Iterable<Node> between(Node start, Node end) {
            return new NodeIterable(start.next, end);
        }

        public class NodeIterable : GLib.Object, Gee.Traversable<Node>, Gee.Iterable<Node> {

            private Node first;
            private Node end;

            public NodeIterable(Node first, Node end) {
                this.first = first;
                this.end = end;
            }

            public Gee.Iterator<Node> iterator() {
                return new NodeIterator(first, end);
            }

            public bool foreach(Gee.ForallFunc<Node> f) {
                return iterator().foreach(f);
            }
        }

        public class NodeIterator : GLib.Object, Gee.Traversable<Node>, Gee.Iterator<Node> {

            public bool read_only { get { return true; } }
            public bool valid { get { return _valid; } }
            private bool _valid;
            private bool is_new;

            private Node node;
            private Node first;
            private Node end;

            public NodeIterator(Node first, Node end) {
                this.first = first;
                this.end = end;
                _valid = false;
                is_new = true;
            }

            public bool has_next() {
                if (is_new)
                    return true;
                return node != null && node != end;
            }

            public bool next() {
                if (is_new) {
                    _valid = true;
                    is_new = false;
                    node = first;
                    return true;
                }
                node = node.next;
                if (node == null || node == end) {
                    _valid = false;
                    return false;
                }
                return true;
            }

            public void remove() {
                GLib.warning("Not implemented");
            }

            public new Node get() {
                return node;
            }

            public bool foreach(Gee.ForallFunc<Node> f) {
                Node t = node;
                while (t != end) {
                    if (!f(t))
                        return false;
                    t = t.next;
                }
                return true;
            }
        }
    }
}
