namespace Commonmark {

    public class ParagraphParser : AbstractBlockParser {

        private Paragraph block = new Paragraph();
        private LinkReferenceDefinitionParser
            link_reference_definition_parser =
            new LinkReferenceDefinitionParser();

        public override bool can_have_lazy_continuation_lines() {
            return true;
        }

        public override Block get_block() {
            return block;
        }

        public override BlockContinue
            try_continue(ParserState state) {
            if (!state.is_blank())
                return BlockContinue.at_index(state.get_index());
            else
                return BlockContinue.none();
        }

        public override void add_line(SourceLine line) {
            link_reference_definition_parser.parse(line);
        }

        public override void add_source_span(SourceSpan source_span) {
            /* Some source spans might belong to link reference definitions,
               others to the paragraph. The parser will handle that. */
            link_reference_definition_parser.add_source_span(source_span);
        }

        public override void close_block() {
            if (link_reference_definition_parser.get_paragraph_lines()
                .is_empty) {
                block.unlink();
            } else {
                block.source_spans = link_reference_definition_parser
                    .get_paragraph_source_spans();
            }
        }

        public override void parse_inlines(InlineParser inline_parser) {
            var lines = link_reference_definition_parser.get_paragraph_lines();
             if (!lines.is_empty)
                 inline_parser.parse(lines, block);
        }

        public SourceLines get_paragraph_lines() {
            return link_reference_definition_parser.get_paragraph_lines();
        }

        public Gee.List<LinkReferenceDefinition> get_definitions() {
            return link_reference_definition_parser.get_definitions();
        }
    }
}
