namespace Commonmark {

    public interface ParserState : GLib.Object {

        /**
         * @return the current source line being parsed (full line)
         */
        public abstract SourceLine get_line();

        /**
         * @return the current index within the line (0-based)
         */
        public abstract int get_index();

        /**
         * @return the index of the next non-space character starting from
         * get_index() (may be the same) (0-based)
         */
        public abstract int get_next_non_space_index();

        /**
         * The column is the position within the line after tab characters have
         * been processed as 4-space tab stops.  If the line doesn't contain any
         * tabs, it's the same as the get_index(). If the line starts with a
         * tab, followed by text, then the column for the first character of the
         * text is 4 (the index is 1)
         *
         * @return the current column within the line (0-based)
         */
        public abstract int get_column();

        /**
         * @return the indentation in columns (either by spaces or tab stop of
         * 4), starting from get_column()
         */
        public abstract int get_indent();

        /**
         * @return true if the current line is blank starting from the index
         */
        public abstract bool is_blank();

        /**
         * @return the deepest open block parser
         */
        public abstract BlockParser get_active_block_parser();
    }
}
