namespace Commonmark {

    /**
     * A set of lines (SourceLine) from the input source.
     */
    public class SourceLines : GLib.Object {

        public Gee.List<SourceLine> lines { get; private set; }

        construct {
            lines = new Gee.ArrayList<SourceLine>();
        }

        public static SourceLines empty() {
            return new SourceLines();
        }

        public static SourceLines from_source_line(SourceLine source_line) {
            var source_lines = new SourceLines();
            source_lines.add_line(source_line);
            return source_lines;
        }

        public static SourceLines from_source_lines(Gee.List<SourceLine> source_lines) {
            var r = new SourceLines();
            r.lines.add_all(source_lines);
            return r;
        }

        public void add_line(SourceLine source_line) {
            lines.add(source_line);
        }

        public bool is_empty { get { return lines.is_empty; } }

        public string get_content() {
            var sb = new StringBuilder();
            foreach (var line in lines) {
                if (sb.len != 0)
                    sb.append_c('\n');
                sb.append(line.content);
            }
            return sb.str;
        }

        public Gee.List<SourceSpan> get_source_spans() {
            var source_spans = new Gee.ArrayList<SourceSpan>();
            foreach (var line in lines) {
                var source_span = line.source_span;
                if (source_span != null) {
                    source_spans.add(source_span);
                }
            }
            return source_spans;
        }
    }
}
