namespace Commonmark {

    public class TextContentWriter : GLib.Object {

        private GLib.OutputStream buffer;
        private unichar last_char;

        public TextContentWriter(GLib.OutputStream _out) {
            buffer = _out;
        }

        public void whitespace() {
            if (last_char != 0 && last_char != ' ') {
                append(" ");
            }
        }

        public void colon() {
            if (last_char != 0 && last_char != ':') {
                append(":");
            }
        }

        public void line() {
            if (last_char != 0 && last_char != '\n') {
                append("\n");
            }
        }

        public void write_stripped(string s) {
            append(s.replace("\r",  " ")
                   .replace("\n", " ")
                   .replace("\t", " "));
        }

        public void write(string s) {
            append(s);
        }

        public void write_char(unichar c) {
            append_char(c);
        }

        private void append(string s) {
            try {
                buffer.write(s.data);
            } catch (GLib.Error e) {
                GLib.error(e.message);
            }

            if (s.length != 0)
                last_char = s.get(s.length - 1);
        }

        private void append_char(unichar c) {
            try {
                uint8[] b = {
                    (uint8)((c & 0xFF000000) >> 24),
                    (uint8)((c & 0x00FF0000) >> 16),
                    (uint8)((c & 0x0000FF00) >>  8),
                    (uint8)((c & 0x000000FF) >>  0)
                };
                buffer.write(b);
            } catch (GLib.Error e) {
                GLib.error(e.message);
            }

            last_char = c;
        }
    }
}
