namespace Commonmark {

    public class ThematicBreakParser : AbstractBlockParser {

        private ThematicBreak block;

        construct {
            block = new ThematicBreak();
        }

        public override Block get_block() {
            return block;
        }

        public override BlockContinue try_continue(ParserState state) {
            /* a horizontal rule can never container > 1 line, so fail to
               match */
            return BlockContinue.none();
        }

        public class Factory : AbstractBlockParserFactory {

            public override BlockStart try_start(ParserState state,
                                                 MatchedBlockParser mbp) {
                if (state.get_indent() >= 4) {
                    return BlockStart.none();
                }
                int next_non_space = state.get_next_non_space_index();
                var line = state.get_line().content;
                if (is_thematic_break(line, next_non_space)) {
                    return BlockStart.from(
                        new ThematicBreakParser()).at_index(line.length);
                } else {
                    return BlockStart.none();
                }
            }
        }

        /* spec: A line consisting of 0-3 spaces of indentation, followed by a
           sequence of three or more matching -, _, or * characters, each
           followed optionally by any number of spaces, forms a thematic
           break. */
        private static bool is_thematic_break(string line, int index) {
            int dashes = 0;
            int underscores = 0;
            int asterisks = 0;
            int length = line.length;
            for (int i = index; i < length; i++) {
                switch (line.get(i)) {
                case '-':
                    dashes++;
                    break;
                case '_':
                    underscores++;
                    break;
                case '*':
                    asterisks++;
                    break;
                case ' ':
                case '\t':
                    /* Allowed, even between markers */
                    break;
                default:
                    return false;
                }
            }

            return ((dashes >= 3 && underscores == 0 && asterisks == 0) ||
                    (underscores >= 3 && dashes == 0 && asterisks == 0) ||
                    (asterisks >= 3 && dashes == 0 && underscores == 0));
        }
    }
}
